(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? factory(require('worker-factory'), require('synchsafe'), require('@babel/runtime/helpers/slicedToArray')) :
    typeof define === 'function' && define.amd ? define(['worker-factory', 'synchsafe', '@babel/runtime/helpers/slicedToArray'], factory) :
    (global = typeof globalThis !== 'undefined' ? globalThis : global || self, factory(global.workerFactory, global.synchsafe, global._slicedToArray));
})(this, (function (workerFactory, synchsafe, _slicedToArray) { 'use strict';

    var textDecoder = null;
    var decode = function decode(dataView) {
      if ('TextDecoder' in self) {
        if (textDecoder === null) {
          textDecoder = new TextDecoder();
        }
        return textDecoder.decode(dataView);
      }
      // @todo Add an expectation test to check if this fallback is still necessary.
      // String.fromCharCode() does normally expect numbers but it can also handle a typed array.
      return String.fromCharCode.apply(null, new Uint8Array(dataView.buffer, dataView.byteOffset, dataView.byteLength));
    };

    var locate = function locate(arrayBuffer) {
      var locations = [];
      var dataView = new DataView(arrayBuffer, 0, 4);
      if (decode(dataView) === 'fLaC') {
        var isLast = false;
        var length = 0;
        var offset = 4;
        while (!isLast) {
          offset += length;
          dataView = new DataView(arrayBuffer, offset, 4);
          isLast = (dataView.getUint8(0) & 0x80) !== 0; // tslint:disable-line:no-bitwise
          length = (dataView.getUint8(3) | dataView.getUint8(2) << 8 | dataView.getUint8(1) << 16) + 4; // tslint:disable-line:max-line-length no-bitwise
        }
        locations.push([0, offset + length]);
      }
      dataView = new DataView(arrayBuffer, 4, 4);
      if (decode(dataView) === 'ftyp') {
        var _offset = 0;
        while (_offset < arrayBuffer.byteLength) {
          dataView = new DataView(arrayBuffer, _offset, 4);
          var _length = dataView.getUint32(0);
          dataView = new DataView(arrayBuffer, _offset + 4, 4);
          var atom = decode(dataView);
          if (atom === 'moov' || atom === 'wide') {
            locations.push([_offset, _offset + _length]);
          }
          _offset += _length;
        }
      }
      dataView = new DataView(arrayBuffer, 0, 3);
      if (decode(dataView) === 'ID3') {
        dataView = new DataView(arrayBuffer, 6, 4);
        locations.push([0, synchsafe.decode(dataView.getUint32(0)) + 10]);
      }
      dataView = new DataView(arrayBuffer, 0, 4);
      if (decode(dataView) === 'OggS') {
        var _offset2 = 0;
        dataView = new DataView(arrayBuffer, 4, 1);
        var streamStructureVersion = dataView.getUint8(0);
        while (streamStructureVersion === 0 && _offset2 < arrayBuffer.byteLength) {
          dataView = new DataView(arrayBuffer, _offset2 + 5, 22);
          /* @todo Make sure the headerTypeFlag is not indicating that this is the first or last
           * page. If so the surrounding pages would need to get an updated headerTypeFlag when
           * stripping the metadata.
           * headerTypeFlag = dataView.getUint8(0);
           */
          var pageSegments = dataView.getUint8(21);
          dataView = new DataView(arrayBuffer, _offset2 + 27, pageSegments + 1);
          var pageSize = 27 + pageSegments;
          for (var i = 0; i < pageSegments; i += 1) {
            pageSize += dataView.getUint8(i);
          }
          var firstByte = dataView.getUint8(pageSegments);
          if (firstByte === 3) {
            dataView = new DataView(arrayBuffer, _offset2 + 27 + pageSegments + 1, 6);
            var identifier = decode(dataView);
            if (identifier === 'vorbis') {
              locations.push([_offset2, _offset2 + pageSize]);
            }
          }
          _offset2 += pageSize;
        }
      }
      dataView = new DataView(arrayBuffer, arrayBuffer.byteLength - 128, 3);
      if (decode(dataView) === 'TAG') {
        locations.push([arrayBuffer.byteLength - 128, arrayBuffer.byteLength]);
      }
      return locations;
    };

    var concat = function concat() {
      for (var _len = arguments.length, arrayBuffers = new Array(_len), _key = 0; _key < _len; _key++) {
        arrayBuffers[_key] = arguments[_key];
      }
      return arrayBuffers.reduce(function (_ref, arrayBuffer) {
        var array = _ref.array,
          offset = _ref.offset;
        array.set(new Uint8Array(arrayBuffer), offset);
        return {
          array: array,
          offset: offset + arrayBuffer.byteLength
        };
      }, {
        array: new Uint8Array(arrayBuffers.reduce(function (byteLength, arrayBuffer) {
          return byteLength + arrayBuffer.byteLength;
        }, 0)),
        offset: 0
      }).array.buffer;
    };

    var strip = function strip(arrayBuffer) {
      var locations = locate(arrayBuffer);
      var offset = 0;
      var strippedArrayBuffer = arrayBuffer;
      locations.forEach(function (_ref) {
        var _ref2 = _slicedToArray(_ref, 2),
          begin = _ref2[0],
          end = _ref2[1];
        if (begin === offset) {
          strippedArrayBuffer = strippedArrayBuffer.slice(end, strippedArrayBuffer.byteLength);
        } else if (end - offset === strippedArrayBuffer.byteLength) {
          strippedArrayBuffer = strippedArrayBuffer.slice(0, begin - offset);
        } else {
          strippedArrayBuffer = concat(strippedArrayBuffer.slice(0, begin - offset), strippedArrayBuffer.slice(end - offset, strippedArrayBuffer.byteLength));
        }
        offset += end - begin;
      });
      return strippedArrayBuffer;
    };

    workerFactory.createWorker(self, {
      locate: function locate$1(_ref) {
        var arrayBuffer = _ref.arrayBuffer;
        return {
          result: locate(arrayBuffer)
        };
      },
      strip: function strip$1(_ref2) {
        var arrayBuffer = _ref2.arrayBuffer;
        var strippedArrayBuffer = strip(arrayBuffer);
        return {
          result: strippedArrayBuffer,
          transferables: [strippedArrayBuffer]
        };
      }
    });

}));
